/*-----------------------------------------------------------------------

File  : chat_client.c

Author: Stephan Schulz

Contents
 
  Simple, telnet-like client for the CSC322 chat system.

  Copyright 1998-2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue Nov 19 01:15:52 CET 2002
    New

-----------------------------------------------------------------------*/

#include "chatlib.h"


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
   int    sock_fd;
   char   buff[DEFAULT_MSG_LEN];
   struct sockaddr_in server_addr;
   int    res;
   fd_set readfds;
   
   if(argc!=4)
   {
      Error("Usage: chat_client <addr> <port> <nickname>");
   }
   sock_fd = socket(PF_INET, SOCK_STREAM, 0);
   if(sock_fd < 0)
   {
      SysErr("socket");      
   }
   memset(&server_addr, 0, sizeof(server_addr));
   server_addr.sin_family = AF_INET;
   server_addr.sin_port = htons(atoi(argv[2]));
   res = inet_pton(AF_INET, argv[1], &server_addr.sin_addr);
   if(res < 0)
   {
      SysErr("inet_pton");
   }
   if(res == 0)
   {
      Error("Not a valid IP address");
   }
   res = connect(sock_fd, (struct sockaddr *) &server_addr,
		 sizeof(server_addr));
   if(res == -1)
   {
      SysErr("connect");
   }
   SecureWrite(STDIN_FILENO, "Connected, type ^D to disconnect.\n");
   
   while(1)
   {
      FD_ZERO(&readfds);
      FD_SET(STDIN_FILENO, &readfds);
      FD_SET(sock_fd, &readfds);
      res = select(4, &readfds, NULL, NULL, NULL);
      if(res == -1)
      {
	 SysErr("select");              
      }
      if(FD_ISSET(STDIN_FILENO, &readfds))
      {
	 res = read(STDIN_FILENO,buff,DEFAULT_MSG_LEN);
	 if(res > 1)
	 {
	    SecureWrite(sock_fd, argv[3]);
	    SecureWrite(sock_fd, ": ");
	    write(sock_fd, buff, res);
	 }
	 else
	 {
	    break;
	 }
      }
      if(FD_ISSET(sock_fd, &readfds))
      {
	 res = read(sock_fd,buff,DEFAULT_MSG_LEN);
	 if(res > 1)
	 {
	    write(STDOUT_FILENO, buff, res);
	 }
      }
   }
   close(sock_fd);
   return EXIT_SUCCESS;
}




/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/



