/*-----------------------------------------------------------------------

File  : print_server.c

Author: Stephan Schulz

Contents
 
  A server that accepts connections and prints incoming data onto
  stdout. Single argument is the port to listen to.

  Copyright 2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Aug 23 22:15:33 CEST 2002
    New

-----------------------------------------------------------------------*/

#include <signal.h>
#include "chatlib.h"

/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

volatile sig_atomic_t sig_int_flag = 0;
SockAdminCell connections[MAX_CONNECTIONS];


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/

typedef void (*SigHandler)(int);

void EstablishSignal(int sig, SigHandler handler)
{
   SigHandler res;
      
   res = signal(sig, handler);
   if(res == SIG_ERR)
   {
      SysErr("signal");
   }   
}

void sig_int_handler(int sig)
{
   assert(sig == SIGINT);
   sig_int_flag = 1;
}


void CloseConnections(void)
{
   int i;
   
   for(i=0; i< MAX_CONNECTIONS; i++)
   {
      if(connections[i].state == PSListening)
      {
	 printf("Closing down listening port.\n");
	 close(connections[i].sock_fd);
      }
      else if(connections[i].state == PSConnected)
      {
	 SecureWrite(connections[i].sock_fd,
		     "Server closing down, disconnecting.\n");
	 close(connections[i].sock_fd);	 
      }
   }
}
   
 /*---------------------------------------------------------------------*/
 /*                         Exported Functions                          */
 /*---------------------------------------------------------------------*/

 int main(int argc, char* argv[])
 {
    int l_index, select_limit,i, res;
    fd_set readfds;

    if(argc!=2)
    {
       Error("Usage: chat_server <port>");
    }
    atexit(CloseConnections);
    EstablishSignal(SIGINT, sig_int_handler);
    EstablishSignal(SIGPIPE, SIG_IGN);

    ConnectionArrayInitialize(connections);
    l_index = ConnectionArrayFindUnused(connections);
    assert(l_index != -1);
    CreateListener(atoi(argv[1]),&(connections[l_index]));
    
    while(!sig_int_flag)
    {
       select_limit = SelectSetInit(connections, &readfds);
       res = select(select_limit, &readfds, NULL, NULL, NULL);
       if(res == -1)
       {
	  SysErr("select");              
       }
       for(i=0; i<MAX_CONNECTIONS; i++)
       {
	  if(connections[i].state != PSUnused && 
	     FD_ISSET(connections[i].sock_fd,&readfds))
	  {
	     switch(connections[i].state)
	     {
	     case PSListening:
		   CreateNewConnection(connections, i);
		   break;
	     case PSConnected:
		   HandleMessage(connections, i);
		   break;
	     default:
		   assert(0 && "Impossible State!\n");
		   break;
	     }
	  }
       }
       
    }
    return EXIT_SUCCESS;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/












