/*-----------------------------------------------------------------------

File  : chatlib.h

Author: Stephan Schulz

Contents

  Part of a small library for a chat client/server system.
 
  Copyright 1998-2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.

Changes

<1> Mon Nov 18 01:49:23 CET 2002
    New

-----------------------------------------------------------------------*/

#ifndef CHATLIB

#define CHATLIB

#include <assert.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>       
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <string.h>
#include "utilities.h"

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define MAX_CONNECTIONS 128
#define DEFAULT_MSG_LEN 256

typedef enum 
{
   PSUnused,
   PSConnected,
   PSListening
}ProtocolState;

typedef struct sock_admin_cell
{
   int                sock_fd;
   ProtocolState      state;
   struct sockaddr_in sock_name;
}SockAdminCell,*SockAdmin_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

int  SecureWrite(int fd, char* message);
int  CreateListener(int port, SockAdmin_p sock_cell);
void ConnectionArrayInitialize(SockAdmin_p array);
int  ConnectionArrayFind(SockAdmin_p array, ProtocolState state);
#define ConnectionArrayFindUnused(array) \
        ConnectionArrayFind((array),PSUnused)
int  SelectSetInit(SockAdmin_p array, fd_set *set);
int  CreateNewConnection(SockAdmin_p connections, int l_index);
void HandleMessage(SockAdmin_p connections, int c_index);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





