/*-----------------------------------------------------------------------

File  : print_server.c

Author: Stephan Schulz

Contents
 
  A server that accepts connections and prints incoming data onto
  stdout. Single argument is the port to listen to.

  Copyright 2002 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri Aug 23 22:15:33 CEST 2002
    New

-----------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>


/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/

void err_exit(char* message)
{
   fprintf(stderr, "print_server: %s\n", message);
   exit(EXIT_FAILURE);
}

void syserr_exit(char* message)
{
   perror(message);
   exit(EXIT_FAILURE);
}


/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/

int main(int argc, char* argv[])
{
   int sock, con_sock;
   struct sockaddr_in sock_name;
   struct sockaddr_in client_sock_name;
   socklen_t client_socl_len;
   char read_buffer[74];
   int bytes_read, i;

   if(argc!=2)
   {
      err_exit("Usage: print_server <port>");
   }
   sock = socket(PF_INET, SOCK_STREAM, 0);
   if(sock == -1)
   {
      syserr_exit("print_server socket");
   }
   sock_name.sin_family = AF_INET;
   sock_name.sin_port = htons(atoi(argv[1]));
   sock_name.sin_addr.s_addr = htonl (INADDR_ANY);
   if (bind(sock, (struct sockaddr *) &sock_name, sizeof(sock_name)) < 0)
   {
      syserr_exit("print-server bind");
   }
   if(listen(sock, 1) == -1)
   {
      syserr_exit("print-server listen");
   }
   con_sock = accept(sock, (struct sockaddr *)&client_sock_name,
		     &client_socl_len);
   if(con_sock == -1)
   {
      syserr_exit("print-server accept");
   }
   write(con_sock, "Hiho and welcome!\n", strlen("Hiho and welcome!\n"));
   while(1)
   {
      bytes_read = read(con_sock, &read_buffer, 72);
      if(bytes_read == -1)
      {
	 syserr_exit("print-server read");
      }
      read_buffer[bytes_read] = 0;
      for(i=bytes_read-1; i>=0; i--)
      {
	 if(isspace(read_buffer[i])||iscntrl(read_buffer[i]))
	 {
	    read_buffer[i] = 0;
	 }
	 else
	 {
	    break;
	 }
      }
      printf("Received: %s\n", read_buffer);

      if(strcmp(read_buffer, "quit")==0)
      {
	 break;
      }
   }
   if(close(con_sock) == -1)
   {
      syserr_exit("print-server close(con_sock)");
   }
   if(close(sock) == -1)
   {
      syserr_exit("print-server close(sock)");
   }
   
   printf("Press <enter> to continue.\n");
   fgetc(stdin);
   
   return EXIT_SUCCESS;
}


/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/












